/*
 * fc-config-override
 * Fontconfig Configuration Override
 *
 * This preloaded wrapper library allows the use of an
 * alternate Fontconfig configuration file on a per-application
 * basis in cases where setting the FC_CONFIG_FILE environment
 * variable doesn't work.  I use this to force the use of
 * non-antialiased bitmap fonts in the GNOME Terminal.
 *
 * Usage:
 *   LD_PRELOAD=/path/to/fc-config-override.so your_application
 *
 * This program is in the public domain - do whatever you want with it.
 *
 * David Simmons
 * August 24, 2009
 *
 */

#include <dlfcn.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#define OLD_PATH "/etc/fonts/fonts.conf"
#define NEW_PATH "/etc/fonts/fonts-bitmap.conf"

int open(const char *pathname, int flags, ...) {
	va_list ap;
	int use_mode = 0;
	mode_t mode = 0;
	int debug = (getenv("FC_WRAPPER_DEBUG")==NULL)?0:1;

	/* remove the LD_PRELOAD environment variable to
	   prevent the wrapper from being mapped into
	   child processes. */
	unsetenv("LD_PRELOAD");

	/* determine if the "mode" argument is in use */
	if (flags & O_CREAT) {
		use_mode = 1;
		va_start(ap, flags);
		mode = va_arg(ap, mode_t);
		va_end(ap);
	}

	/* debug? */
	if (debug) {
		printf("\twrapper: \"%s\" flags=%08lX",
			pathname, (unsigned long)flags);
		if (use_mode)
			printf(" mode=0%o", mode);
		printf("\n");
	}

	/* lookup the original open function */
	typedef int (*open_t)(const char*,int,...);
	open_t libc_open = dlsym(((void *)-1L), "open");

	/* substitute the new path? */
	if (strcmp(pathname, OLD_PATH) == 0) {
		pathname = NEW_PATH;
		if (debug)
			printf("\tsubstituting path: \"%s\"\n", pathname);
	}

	/* dispatch to the libc open() function */
	if (use_mode) {
		return libc_open(pathname, flags, mode);
	} else {
		return libc_open(pathname, flags);
	}
}
 
